@@ -15,7 +15,7 @@ import xml2rfc.util
 import xml2rfc.utils
 
 from lxml import etree
-from optparse import Values
+from argparse import Namespace
 
 try:
     from xml2rfc import debug
@@ -35,7 +35,9 @@ default_silenced_messages = [
 #    ".*[Pp]ostal address",
 ]
 
-default_options = Values(defaults={
+
+default_options = Namespace()
+default_options.__dict__ = {
         'accept_prepped': None,
         'add_xinclude': None,
         'basename': None,
@@ -43,6 +45,8 @@ default_options = Values(defaults={
         'cache': None,
         'clear_cache': False,
         'css': None,
+        'config_file': None,
+        'country_help': False,
         'date': datetime.date.today(),
         'datestring': None,
         'debug': False,
@@ -52,7 +56,7 @@ default_options = Values(defaults={
         'dtd': None,
         'expand': False,
         'external_css': False,
-        'external_js': True,            # Change to False in 3.x
+        'external_js': False,
         'filename': None,
         'first_page_author_org': True,
         'html': False,
@@ -62,8 +66,8 @@ default_options = Values(defaults={
         'image_svg': False,
         'indent': 2,
         'info': False,
-        'legacy': True,
-        'legacy_date_format': False,    # Change to True in 3.x
+        'legacy': False,
+        'legacy_date_format': False,
         'legacy_list_symbols': False,
         'list_symbols': ('*', '-', 'o', '+'),
         'manpage': False,
@@ -77,7 +81,9 @@ default_options = Values(defaults={
         'output_filename': None,
         'output_path': None,
         'pagination': True,
+        'pi_help': False,
         'pdf': False,
+        'pdf_help': False,
         'preptool': False,
         'quiet': False,
         'remove_pis': False,
@@ -86,6 +92,8 @@ default_options = Values(defaults={
         'rfc_base_url': 'https://www.rfc-editor.org/rfc/',
         'rfc_reference_base_url': 'https://rfc-editor.org/rfc/',
         'silence': default_silenced_messages,
+        'skip_config_files': False,
+        'source': None,
         'strict': False,
         'table_hyphen_breaks': False,
         'table_borders': 'full',
@@ -93,12 +101,14 @@ default_options = Values(defaults={
         'text': True,
         'unprep': False,
         'utf8': False,
+        'values': False,
         'verbose': False,
         'version': False,
         'v2v3': False,
+        'v3': True,
         'vocabulary': 'v2',
         'widows': 2,
-    })
+    }
 
 class _RfcItem:
     """ A unique ID object for an anchored RFC element.
@@ -1687,8 +1697,9 @@ def get_xref_tags():
 element_tags= get_element_tags()
 meta_tags   = get_meta_tags() - deprecated_element_tags
 text_tags   = get_text_tags() - deprecated_element_tags
-inline_tags = get_inline_tags() - deprecated_element_tags
-block_tags  = element_tags - inline_tags - deprecated_element_tags
+all_inline_tags = get_inline_tags()
+inline_tags = all_inline_tags - deprecated_element_tags
+block_tags  = element_tags - inline_tags - deprecated_element_tags - set(['t'])
 mixed_tags  = get_text_or_block_tags() - deprecated_element_tags
 xref_tags   = get_xref_tags()
 
@@ -1959,7 +1970,9 @@ class BaseV3Writer(object):
     def page_top_center(self):
         title = self.root.find('./front/title')
         text = title.get('abbrev') or ' '.join(title.itertext())
-        return text
+        if len(text) > 40:
+            self.warn(title, "Expected a title abbreviation of not more than 40 character for the page header, found %s characters" % len(text))
+        return text[:40]
 
     def page_top_right(self):
         date = self.root.find('./front/date')
@@ -2083,14 +2096,14 @@ class BaseV3Writer(object):
             for c in e:
                 indent(c, i+ind)
             #
-            if e.tag not in inline_tags:# and e.tag not in ['artwork', 'sourcecode', ]:
+            if e.tag not in all_inline_tags:# and e.tag not in ['artwork', 'sourcecode', ]:
                 if e.tail is None or e.tail.strip()=='':
                     if e.getnext() != None:
                         e.tail = '\n'+' '*i
                     else:
                         e.tail = '\n'+' '*(i-ind)
                 else:
-                    self.warn(e, 'Unexpected tail: <%s>%s' % (e.tag, e.tail))
+                    self.warn(e, 'Unexpected text content after block tag: <%s>%s' % (e.tag, e.tail))
             else:
                 if e.tail != None and e.tail.strip()=='' and '\n' in e.tail:
                     if e.getnext() != None:
